<?php

namespace Drupal\outvoice\Api;

/**
 * Class for connecting to the OutVoice API.
 */
class OutvoiceApi {

  /**
   * The OutVoice API address.
   *
   * @var string
   */
  protected $apiUrl = 'https://api.outvoice.com/';

  /**
   * The OutVoice API Version.
   *
   * @var string
   */
  protected $apiVersion = 'v1.1/';

  /**
   * The OutVoice module version.
   *
   * @var string
   */
  protected $moduleVersion = 'drupal7.x-1.0';

  /**
   * The OutVoice Client ID.
   *
   * @var string
   */
  protected $clientID = '08a3b847-025c-40a9-bfc8-4ed891286f99';

  /**
   * OAuth Access Token.
   *
   * @var string
   */
  public $accessToken;

  /**
   * OAuth Refresh Token.
   *
   * @var string
   */
  public $refreshToken;

  /**
   * OAuth token expires.
   *
   * @var string
   */
  public $tokenExpires;

  /**
   * Generates OAuth tokens from username and password.
   *
   * @param string $username
   *   User's OutVoice username.
   * @param string $password
   *   User's OutVoice password.
   *
   * @return bool
   *   Returns TRUE if successful.
   */
  public function generateTokens($username, $password) {

    $data = array(
      'grant_type' => 'password',
      'client_id' => $this->clientID,
      'username' => $username,
      'password' => $password,
    );
    $options = array(
      'method' => 'POST',
      'data' => http_build_query($data),
      'timeout' => 15,
      'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
    );
    $client = drupal_http_request($this->apiUrl . "oauth/token", $options);
    $tokens = json_decode($client->data);
    if (isset($tokens->access_token)) {
      $this->accessToken = $tokens->access_token;
      $this->refreshToken = $tokens->refresh_token;
      $this->tokenExpires = time() + 3000;
      return TRUE;
    }
    return FALSE;

  }

  /**
   * Refreshes OAuth tokens.
   *
   * @return bool
   *   Returns TRUE if successful.
   */
  public function refreshTokens() {

    $data = array(
      'grant_type' => 'refresh_token',
      'client_id' => $this->clientID,
      'refresh_token' => $this->refreshToken,
    );
    $options = array(
      'method' => 'POST',
      'data' => http_build_query($data),
      'timeout' => 15,
      'headers' => array('Content-Type' => 'application/x-www-form-urlencoded'),
    );
    $client = drupal_http_request($this->apiUrl . "oauth/token", $options);

    $tokens = json_decode($client->data);
    if (isset($tokens->access_token)) {
      $this->accessToken = $tokens->access_token;
      $this->refreshToken = $tokens->refresh_token;
      $this->tokenExpires = time() + 3000;
      return TRUE;
    }
    return FALSE;

  }

  /**
   * Sets OAuth tokens.
   *
   * @param string $access
   *   OAuth access token.
   * @param string $refresh
   *   OAuth refresh token.
   */
  public function setTokens($access, $refresh) {
    $this->accessToken = $access;
    $this->refreshToken = $refresh;
  }

  /**
   * Retrieves access token and refreshes it if needed.
   */
  public function getAccessToken() {
    if (time() > $this->tokenExpires) {
      $this->refreshTokens();
    }
    return $this->accessToken;
  }

  /**
   * Retrieves refresh token.
   */
  public function getRefreshToken() {
    return $this->refreshToken;
  }

  /**
   * Handles an OutVoice payment.
   *
   * @param array $info
   *   An array of values for the payment.
   *     - freelancer: The ID of the contributor.
   *     - amount: The payment amount.
   *     - currency: Payment currency.
   *     - freelancer1: ID of second contributor (optional).
   *     - amount1: The second payment amount (optional).
   *     - url: The URL of the content published.
   *     - title: The title of the content published.
   *
   * @return string
   *   Returns message in plain text.
   */
  public function payment(array $info) {

    $data = array(
        0 => array(
          "freelancer" => $info['freelancer'],
          "amount"     => self::outvoiceFormatAmount($info['amount']),
          "currency"   => $info['currency'],
          "url"        => $info['url'],
          "title"      => $info['title'],
          "version"    => $this->moduleVersion,
        ),
    );
    // Add second contributor.
    if (!empty($info['amount1'])) {
      $data[1] = [
        "freelancer" => $info['freelancer1'],
        "amount"     => self::outvoiceFormatAmount($info['amount1']),
        "currency"   => $info['currency1'],
        "url"        => $info['url'],
        "title"      => $info['title'],
        "version"    => $this->moduleVersion,
      ];
    }
    $data = json_encode($data);

    $options = array(
      'method' => 'POST',
      'data' => $data,
      'timeout' => 15,
      'headers' => array(
        'Content-Type' => 'application/json',
        'Authorization' => 'Bearer ' . $this->getAccessToken()
        ),
    );

    $request = drupal_http_request($this->apiUrl . $this->apiVersion . "transaction", $options);
    $message = "There was an error. Please log into your OutVoice account to confirm payment.";
    if ($request->code == 200) {
      $message = $request->data;
    }

    return $message;

  }

  /**
   * List of all contributors.
   *
   * @return array
   *   Returns an array of contributors keyed by their IDs
   */
  public function listContributors() {

    $options = array(
      'method' => 'GET',
      'timeout' => 15,
      'headers' => array('Authorization' => 'Bearer ' . $this->getAccessToken()),
    );
    $client = drupal_http_request($this->apiUrl . $this->apiVersion . "list-freelancers", $options);
    $contributorList = json_decode($client->data,TRUE);
    return $contributorList;

  }

  /**
   * Formats currency amounts.
   *
   * @param int $amount
   *   Number indicating amount to be paid.
   *
   * @return string
   *   Formatted without decimal.
   */
  public static function outvoiceFormatAmount($amount) {

    $decimal = strpos($amount, ".");
    if ($decimal === FALSE) {
      $amount = $amount . "00";
    }
    else {
      // Ensure 2 digits after decimal.
      $exploded = explode(".", $amount);
      if (empty(strlen($exploded[1]))) {
        $amount = $amount . "00";
      }
      if (strlen($exploded[1]) == 1) {
        $amount = $amount . "0";
      }
    }
    return $amount;

  }

}
